class RemovedInWagtailMenus213Warning(DeprecationWarning):
    pass


removed_in_next_version_warning = RemovedInWagtailMenus213Warning


class RemovedInWagtailMenus3Warning(PendingDeprecationWarning):
    pass


class RemovedInWagtailMenus31Warning(PendingDeprecationWarning):
    pass
